% SOLVEPWC  solves PDE with piecewise constant material and source term
%
% function [u, q, K] = solvePWC(x, p, AE, c, f);
%
% x   nodes of the mesh
% p   polynomial degree or polynomial degree per cell
% AE  constant AE or AE per cell
% c   constant c or c per cell
% f   function handle of source term
%
% u   solution vector
% q   vector of right hand side
% K   system matrix
%
function [u, q, K] = solvePWCQuad(x, p, AE, c, f);

% intervall lengthes
h = diff(x); n = length(h);

if n < 2
  disp('At least 2 intervals needed. But only one given.');
  K = []; b = [];
  return;
end

% transform constant material to piecewise constant material
if length(AE) == 1
  AE = AE*ones(1,n);  
end
if length(c) == 1
  c = c*ones(1,n);  
end
% transform constant polynomial degree to polynomial degree per element
if length(p) == 1
  p = p*ones(1,n);  
end
% smallest polynomial degree has to be 1
p(find(p < 1)) = 1;

% number of degrees of freedom
N = (n - 1) + (sum(p) - length(p));
K = sparse(N,N);
q = zeros(N,1);

% reference element matrices
S = refStiff(max(p));
M = refMass (max(p));

% assembling
cnt = n;  % counter for index of interior functions
for j=1:n
  % global and local indices 
  if j == 1
    I = j;       Iloc = 2:1+p(j);
  elseif j==n
    I = j-1;     Iloc = [1 3:1+p(j)]; 
  else
    I = [j-1 j]; Iloc = 1:1+p(j);
  end
  I = [I cnt+[0:p(j)-2]];
  
  % element matrix
  Kloc = 2/h(j)*AE(j)* S(Iloc,Iloc) + ...
	 h(j)/2*c(j) * M(Iloc,Iloc);
  
  % local load vector
  [xi,w] = gauleg(p(j)+10);
  phi    = shapefct1D(p(j), xi);
  f_     = f(x(j) + h(j)*(xi+1)/2);
  qloc   = h(j)/2 * phi(Iloc,:) * (f_ .* w)';
  
  % adding in global matrix and vector of rhs
  K(I,I) = K(I,I) + Kloc;
  q(I)   = q(I)   + qloc;
  % increase counter
  cnt = cnt + p(j)-1;
end

u = K \ q;

return;