% SOLVEPWCSTATCOND  solves PDE with piecewise constant material and source term and use of static condenstation
%
% function [u, q, K] = solvePWCStatCond(x, p, AE, c, f);
%
% x   nodes of the mesh
% p   polynomial degree or polynomial degree per cell
% AE  constant AE or AE per cell
% c   constant c or c per cell
% f   constant f (source) xor f per cell
%
% u   solution vector
% q   vector of right hand side
% E   energy norm
%
function [u, q, E] = solvePWCStatCond(x, p, AE, c, f);

% intervall lengthes
h = diff(x); n = length(h);

if n < 2
  disp('At least 2 intervals needed. But only one given.');
  K = []; b = [];
  return;
end

% transform constant material to piecewise constant material
if length(AE) == 1
  AE = AE*ones(1,n);  
end
if length(c) == 1
  c = c*ones(1,n);  
end
if length(c) == 1
  f = f*ones(1,n);  
end
% transform constant polynomial degree to polynomial degree per element
if length(p) == 1
  p = p*ones(1,n);  
end
% smallest polynomial degree has to be 1
p(find(p < 1)) = 1;

% number of degrees of freedom
N = n - 1;
K = sparse(N,N);
q = zeros(N,1);

% reference element matrices
S = refStiff(max(p));
M = refMass (max(p));
% reference load vector
qref = [1;1;-2/sqrt(6);zeros(max(p)-1,1)];

% assembling
for j=1:n
  % global and local indices 
  if j == 1
    I = j;       Iloc = 2:1+p(j);     I0 = 1;
  elseif j==n
    I = j-1;     Iloc = [1 3:1+p(j)]; I0 = 1;
  else
    I = [j-1 j]; Iloc = 1:1+p(j);     I0 = 1:2;
  end
  I1 = [max(I0)+1:length(Iloc)];
  
  % element matrix
  Kloc = 2/h(j)*AE(j)* S(Iloc,Iloc) + ...
	 h(j)/2*c(j) * M(Iloc,Iloc);
  % element vector of rhs
  qloc = h(j)/2*f(j) * qref(Iloc);
  
  % static condensation
  % Equation (3.2.33), (3.2.34) on p. 68 of Schwab book
  Kq = [Kloc(I0,I0) qloc(I0)];
  if p(j) > 1
    Kq = Kq - Kloc(I0,I1)*full(Kloc(I1,I1) \ [Kloc(I1,I0) qloc(I1)]);
  end

  % adding in global matrix and vector of rhs
  K(I,I) = K(I,I) + Kq(I0,I0);
  q(I)   = q(I)   + Kq(I0,max(I0)+1);
end

% solving the system of equations
u = K \ q;

E = 0;

% computing the internal amplitudes
for j=1:n
  % global and local indices 
  if j == 1
    I = j;       Iloc = 2:1+p(j);     I0 = 1;
  elseif j==n
    I = j-1;     Iloc = [1 3:1+p(j)]; I0 = 1;
  else
    I = [j-1 j]; Iloc = 1:1+p(j);     I0 = 1:2;
  end
  I1 = [max(I0)+1:length(Iloc)];
  
  % element matrix
  Kloc = 2/h(j)*AE(j)* S(Iloc,Iloc) + ...
	 h(j)/2*c(j) * M(Iloc,Iloc);
  % element vector of rhs
  qloc = h(j)/2*f(j) * qref(Iloc);
  
  if p(j) > 1
    uloc = Kloc(I1,I1) \ (qloc(I1) - Kloc(I1,I0)*u(I));
    u = [u; uloc];
  else
    uloc = [];
  end
  uloc = [u(I);uloc];
  
  E = E + uloc' * Kloc * uloc;
end

return;